local AddOn, NS = ...

local strfind = string.find
local ceil = math.ceil

local function SetupClickTargetting(frame, settings)
	local buttons = {"leftclick", "rightclick", "middleclick" }
	local actions = {"target", "assist", "togglemenu"} -- Fizzlemizz: Comment: Fix for focus frame
	for butonval, button in pairs(buttons) do
--		local frameindex = DUF_FRAME_DATA[unit].index
		local buttonindex = settings[button]
		if buttonindex then
			local action = actions[buttonindex]
--			frame:SetAttribute("*type"..butonval, action)
			frame:SetAttribute("type"..butonval, action)
		end
	end
end

local function DUF_Options_LoadDefaultSettings()
	if (not DUF_DEFAULT_SETTINGS) then
		DL_Feedback(DUF_TEXT.NoDefaultSettings)
		return
	end
	DUF_Settings[DUF_INDEX] = {nil}
	DL_Copy_Table(DUF_DEFAULT_SETTINGS, DUF_Settings[DUF_INDEX])
	DUF_Settings[DUF_PLAYER_INDEX] = DUF_INDEX
end

local function DUF_Options_SetUpdateSpeed()
	DUF_Settings[DUF_INDEX].updatespeed = 1 / DUF_Settings[DUF_INDEX].updatespeedbase
end

function NS:SetVisibility(frame, unitClass, hideSetting)
	local _, uc = UnitClass("player")
	if (unitClass == uc or DUF_OPTIONS_VISIBLE) then
		if (not hideSetting) then
			if (not frame:IsVisible()) then
				frame:Show()
			end
		else
			if (frame:IsVisible()) then
				frame:Hide()
			end
		end
	else
		if (frame:IsVisible()) then
			frame:Hide()
		end
	end
end

local function DUF_AltManaBar_UpdateType(StatusBar) 
	NS:DUF_ManaBar_UpdateType(StatusBar, NS.Bar.AltMana)
end

local function DUF_Initialize_BaseSettings(frame, settings, debug)
	local framename = frame:GetName()
	local bgframe = frame.Background
	if not DUF_OPTIONS_VISIBLE then
		frame.reaction = nil
	end
	local bgtexture = "Interface\\AddOns\\DiscordLibrary\\PlainBackdrop"
	if (settings.bgtexture and settings.bgtexture ~= "") then
		bgtexture = NS:GetResourcePath(settings.bgtexture)
	end
	if (settings.backgroundstyle == 1) then
		bgframe:SetBackdrop({bgFile = bgtexture, edgeFile = "Interface\\AddOns\\DiscordLibrary\\PlainBackdrop", tile = nil, tileSize = 1, edgeSize = 1, insets = { left = 1, right = 1, top = 1, bottom = 1 }})
	elseif (settings.backgroundstyle == 2) then
		bgframe:SetBackdrop({bgFile = bgtexture, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = nil, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 5, top = 5, bottom = 5 }})
	elseif (settings.backgroundstyle == 3) then
		bgframe:SetBackdrop({bgFile = bgtexture, edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border", tile = nil, tileSize = 32, edgeSize = 32, insets = { left = 11, right = 12, top = 12, bottom = 11 }})
	elseif (settings.backgroundstyle == 4) then
		bgframe:SetBackdrop({bgFile = bgtexture, edgeFile = "Interface\\Buttons\\UI-SliderBar-Border", tile = nil, tileSize = 8, edgeSize = 8, insets = { left = 3, right = 3, top = 6, bottom = 6 }})
	end
	if settings.bgcolor.r == nil then
		settings.bgcolor.r = 0 -- fix for change in SetVertecColor not accepting nil
		settings.bgcolor.g = 0
		settings.bgcolor.b = 0
	end
	if settings.bordercolor.r == nil then
		settings.bordercolor.r = 0 -- fix for change in SetVertecColor not accepting nil
		settings.bordercolor.g = 0
		settings.bordercolor.b = 0
	end
	bgframe:SetBackdropColor(settings.bgcolor.r, settings.bgcolor.g, settings.bgcolor.b, settings.bgalpha)
	bgframe:SetBackdropBorderColor(settings.bordercolor.r, settings.bordercolor.g, settings.bordercolor.b, settings.borderalpha)
	if (settings.hidebg) then
		bgframe:Hide()
	else
		bgframe:Show()
	end
	frame:ClearAllPoints()
	frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	if (settings.framelevel and settings.framestrata) then
		frame:SetFrameStrata(settings.framestrata)
		if not frame.baseframelevel then
			frame.baseframelevel = frame:GetFrameLevel()
		end
		local framelevel = frame.baseframelevel + settings.framelevel
		if (framelevel < 0) then framelevel = 0 end
		frame:SetFrameLevel(framelevel)
	end
	bgframe:SetFrameStrata("BACKGROUND")
	local height, width
	if (settings.size) then
		height = settings.size
		width = settings.size
	else
		height = settings.height
		width = settings.width
	end
	if (frame.dynamicsize) then
		height = height * frame.dynamicsize
		width = width * frame.dynamicsize
	end
	frame:SetWidth(width)
	frame:SetHeight(height)
	frame:SetAlpha(settings.alpha)
	bgframe:SetWidth(width + settings.bgpadding * 2)
	bgframe:SetHeight(height + settings.bgpadding * 2)
	if (settings.hide) then
		frame:Hide()
	else
		frame:Show()
	end
	if settings.mouseover then
		frame:SetAlpha(0)
	end
	frame.flashing = nil
	if framename == "DUF_ClassFrame" and not DUF_OPTIONS_VISIBLE then
		frame:EnableMouse(false)
		return
	end
	if (settings.disablemouse) then
		frame:EnableMouse(false)
	else
		frame:EnableMouse(1)
	end
end

local function DUF_Initialize_IconBackground(frame, settings)
	frame = frame.Circle
	if (settings.circle) then
		frame:Show()
	else
		frame:Hide()
		return
	end
	local background = frame.CircleBackground
	local border = frame.CircleBorder
	background:SetVertexColor(settings.bgcolor.r, settings.bgcolor.g, settings.bgcolor.b, settings.bgalpha)
	border:SetVertexColor(settings.bordercolor.r, settings.bordercolor.g, settings.bordercolor.b, settings.borderalpha)
end

local function DUF_CreatePortrait(unit) -- Fizzlemizz: Function added to move portraits from XML to allow for 3D portraits
	local use3D, use3Dface = NS:DUF_Get3D(unit)
	local p2d = _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame2d"]
	local p3d = _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame3d"]
	if (use3D) then 
		if (p3d) then
			return
		end
	else
		if (p2d) then
			return 
		end
	end
	local frame = _G[DUF_FRAME_DATA[unit].frame .. "_Portrait"]
	if (frame) then
		frame:Hide()
	end
	if (use3D) then	
		if not _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame3d"] then
			frame = CreateFrame("Button" , "$parent_Portrait" , _G[DUF_FRAME_DATA[unit].frame] , "DUF_3DPortraitTemplate")
			_G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame3d"] = frame
		else
			frame = _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame3d"]
		end
		if _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame2d"] then
			_G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame2d"]:Hide()
		end
	else
		if (not _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame2d"]) then
			frame = CreateFrame("Button" , "$parent_Portrait" , _G[DUF_FRAME_DATA[unit].frame] , "DUF_PortraitTemplate")
			_G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame2d"] = frame
		else
			frame = _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame2d"]
		end
		if _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame3d"] then
			_G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame3d"]:Hide()
		end
	end
	_G[DUF_FRAME_DATA[unit].frame .. "_Portrait"] = frame
	local texture, border = frame:GetRegions()
	_G[DUF_FRAME_DATA[unit].frame.."_Portrait"].PortraitBorder = border
	_G[DUF_FRAME_DATA[unit].frame.."_Portrait"].Portrait = texture
	_G[frame:GetName().."_Border"] = border
	_G[frame:GetName().."_Texture"] = texture
	frame:Show()
end

----------------------------------------------------------------------------------------------------
function NS:DUF_CoreInitialize() -- Fizzlemizz: Comment: Changed name from DUF_Initialize to DUF_CoreInitialize so it's easier to find amongst all the other DUF_Initialize... functions
	if (DUF_INITIALIZED) then return end
	if (NS.DUF_DL_VERSION > DL_VERSION) then
		DL_Debug("** You need to install the latest version of the Discord Library for this mod to work right.  It should be included in the same .zip file from which you extracted this mod. **")
		return
	end
	local unitclass = select(2, UnitClass("player"))
	if unitclass == "DRUID" then
		NS.DUF_ISDRUID = true
	end
	if (not DUF_Settings) then
		DUF_Settings = {}
--[[
	elseif (not DUF_Settings["INITIALIZED2.3"]) then
		for index in pairs(DUF_Settings) do
			if (index ~= "SavedSettings") then
				DUF_Settings[index] = nil
			end
		end
		for profile in pairs(DUF_Settings.SavedSettings) do
			DUF_Settings[profile] = {}
			DL_Copy_Table(DUF_Settings.SavedSettings[profile], DUF_Settings[profile])
		end
		DUF_Settings.SavedSettings = nil
		DUF_Settings["INITIALIZED2.3"] = 1
]]--
	end
	DUF_PLAYER_INDEX = UnitName("player").." :: ".. GetRealmName()
	DUF_INDEX = DUF_Settings[DUF_PLAYER_INDEX]
	if (not DUF_INDEX) then
		DUF_INDEX = DUF_TEXT.Default
		DUF_Settings[DUF_PLAYER_INDEX] = DUF_INDEX
	end

	if (not DUF_Settings[DUF_INDEX]) then
		DUF_Options_LoadDefaultSettings()
	end

	if (DUF_CUSTOM_SETTINGS) then
		DUF_Settings.Custom = {}
		DL_Copy_Table(DUF_CUSTOM_SETTINGS, DUF_Settings.Custom)
		DUF_CUSTOM_SETTINGS = nil
	end
	NS:DUF_Initialize_NewSettings()
	DUF_Settings.CharacterSettings = nil
	DUF_INITIALIZED = true
	DUF_InitializeDurationUpdates()
	NS:DUF_SetDefaultUIHooks()
	DUF_Initialize_AllFrames()
	DUF_Main_UpdatePartyMembers()
	NS:DUF_XPBar_Update()
	NS:DUF_PetXPBar_Update()
	NS:DUF_HonorBar_Update()
end

function DUF_Initialize_AllFrames(debug) -- May be called by other addons FM
	local unitFrame
	if (not DUF_Settings[DUF_INDEX].rootpath) then
		DUF_Settings[DUF_INDEX].rootpath = ""
	end
	if (not DUF_Settings[DUF_INDEX].savedroots) then
		DUF_Settings[DUF_INDEX].savedroots = {}
	end
	for unit, value in pairs(DUF_FRAME_DATA) do
		local unitFrame = _G[DUF_FRAME_DATA[unit].frame]
		DUF_CreatePortrait(unit)  -- Fizzlemizz: Function called here to create portrait frames before SetUnitFrameWatch to allow for 3D portraits
		DUF_SetFrameUnitWatch(unitFrame, not DUF_Settings[DUF_INDEX][value.index].hide) 
		SetupClickTargetting(unitFrame, DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index])
		local children = { unitFrame:GetChildren() }
		for _, child in ipairs(children) do
			child.menu = unitFrame.menu
		end
		DUF_Initialize_Frame(unitFrame, unit)
		DUF_Initialize_Portrait(unit)
		DUF_Initialize_ClassIcon(unit)
		DUF_Initialize_RaceIcon(unit)
		DUF_Initialize_RankIcon(unit)
		DUF_Initialize_RoleIcon(unit)
		local tb
		if (unit == "player") then
			tb = NS.DUF_NUM_PLAYER_TEXTBOXES
		else
			tb = NS.DUF_NUM_OTHERS_TEXTBOXES
		end
		for i=1,tb do
			DUF_Initialize_TextBox(unit, i) 
		end
		for i=1, NS.DUF_NUM_STATUSBARS do
			DUF_Initialize_StatusBar(unit, i, nil, debug) 
		end
		DUF_Initialize_PVPIcon(unit)
		DUF_Initialize_StatusIcon(unit)
		DUF_Initialize_PhaseIcon(unit)
		DUF_Initialize_LootIcon(unit)
		DUF_Initialize_LeaderIcon(unit)
		DUF_Initialize_Buffs(unitFrame, unit)
		DUF_Initialize_Debuffs(unitFrame, unit)
	end
	DUF_Initialize_TempEnchants()
	DUF_Initialize_HappinessIcon()
	local class = select(2, UnitClass("player"))
	DUF_Initialize_ClassBar(class)
	if TotemFrame then
		DUF_Initialize_TotemFrame()
	end
	DUF_Initialize_TargetMarkers()
	DUF_Initialize_QuestBossIcon()
	DUF_Initialize_EliteTexture()
	if (DUF_Options) then
		DUF_Options_SetScale()
		DUF_Init_ContextColors()
		DUF_Init_MiscOptions()
	end	
	DUF_Options_SetUpdateSpeed()
-- Legion	DUF_Toggle_DefaultFrames()
	DUF_PlayerFrame:Hide()
	if (not DUF_Settings[DUF_INDEX].player.hide) then
		DUF_PlayerFrame:Show()
	end
	for i=1,DL_UNIT_BUFFCOUNT do
		_G["DUF_PlayerFrame_Buffs_"..i].playerbuff = true
		_G["DUF_PlayerFrame_Debuffs_"..i].playerbuff = true
	end
	if (DUF_Settings[DUF_INDEX].hidebuffframe) then
		BuffFrame:Hide()
		BuffFrame.origShow = BuffFrame.Show
		BuffFrame.Show = function() end
	elseif (BuffFrame.origShow) then
		BuffFrame.Show = BuffFrame.origShow
		BuffFrame.origShow = nil
		BuffFrame:Show()
	end
	if DebuffFrame then
		if (DUF_Settings[DUF_INDEX].hidedebuffframe) then
			DebuffFrame:Hide()
			DebuffFrame.origShow = BuffFrame.Show
			DebuffFrame.Show = function() end
		elseif (DebuffFrame.origShow) then
			DebuffFrame.Show = BuffFrame.origShow
			DebuffFrame.origShow = nil
			DebuffFrame:Show()
		end
	end
-- Moving to single eventFrame	DUF_UnitFrame_OnEvent(DUF_TargetFrame, "PLAYER_TARGET_CHANGED")
	if DUF_OPTIONS_VISIBLE then
		DUF_Options_ShowFrames()
	end
end

function DUF_Options_LoadSettings(index, safe, silent)
	if (not index) then
		index = DUF_LOADINDEX
	end
	if (index == "" or (not index)) then
		return
	end
	if (not DUF_Settings[index]) then
		DL_Feedback(DUF_TEXT.NoSettingsFound)
		return
	end
	DUF_Settings[DUF_PLAYER_INDEX] = index
	DUF_INDEX = index
	NS:DUF_Initialize_NewSettings()
	if (safe) then
		ReloadUI()
	else
		DUF_Initialize_AllFrames()
		if not silent then
			DL_Feedback(DUF_TEXT.SettingsLoaded)
		end
	end
	if (DUF_Options) then
		DUF_MiscOptions_CurrentProfile:SetText(DUF_TEXT.CurrentProfile..":  "..DUF_INDEX)
		DUF_MiscOptions_LoadSettings_Setting:SetText("")
	end
end

function DUF_Initialize_TempEnchants()
	if not TemporaryEnchantFrame then -- Dragonflight change
		return
	end
	TemporaryEnchantFrame:Hide()
	local prevname , duf_button
	local frame = _G[DUF_FRAME_DATA["player"].frame.."_TempEnchants"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA["player"].index].TempEnchants
	NS:DUF_Initialize_Element("player", frame, settings)
	frame:SetFrameStrata("BACKGROUND") 
	for i=1,3 do --NUM_TEMP_ENCHANT_FRAMES
--		prevname = duf_button
		duf_button = _G[frame:GetName().."_" .. i]
		duf_button:ClearAllPoints()
		if (i == 1) then
			duf_button:SetPoint("TOPLEFT", frame:GetName(), "TOPLEFT", 0, 0)
		else
			if (settings.vertical) then
--				duf_button:SetPoint("TOP", prevname, "BOTTOM", 0, -settings.vspacing)
				duf_button:SetPoint("TOP", frame.TE[i-1], "BOTTOM", 0, -settings.vspacing)
			else
--				duf_button:SetPoint("LEFT", prevname, "RIGHT", settings.hspacing, 0)
				duf_button:SetPoint("LEFT", frame.TE[i-1], "RIGHT", settings.hspacing, 0)
			end
		end
		duf_button:SetWidth(settings.size)
		duf_button:SetHeight(settings.size)
		local icon = _G[DUF_FRAME_DATA["player"].frame.."_TempEnchants_" .. i .. "_Texture"]
		if (DUF_OPTIONS_VISIBLE) then
			icon:SetTexture("Interface/AddOns/DiscordUnitFrames/Icons/Buff")
			duf_button:Show()
		else
			icon:SetTexture()
			duf_button:Hide()
		end
	end
	TemporaryEnchantFrame:Show() 
end

function DUF_Initialize_Buffs(self, unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_Buffs"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].Buffs
	NS:DUF_Initialize_Element(unit, frame, settings)
	if (settings.reverse) then
		local button = _G[frame:GetName().."_"..DL_UNIT_BUFFCOUNT]
		local buttontexture = _G[frame:GetName().."_"..DL_UNIT_BUFFCOUNT.."_Texture"]
		button:ClearAllPoints()
		button:SetPoint("BOTTOMRIGHT", frame:GetName(), "BOTTOMRIGHT", 0, 0)
		button:SetFrameLevel(frame:GetFrameLevel() + 1)
		button:SetWidth(settings.size)
		button:SetHeight(settings.size)
		buttontexture:SetWidth(settings.size)
		buttontexture:SetHeight(settings.size)
		local prevname
		for i=DL_UNIT_BUFFCOUNT-1, 1, -1 do
			prevname = frame:GetName().."_"..(i+1)
			button = _G[frame:GetName().."_"..i]
			buttontexture = _G[frame:GetName().."_"..i.."_Texture"]
			button:ClearAllPoints()
			if (settings.vertical) then
				button:SetPoint("BOTTOM", prevname, "TOP", 0, settings.vspacing)
			else
				button:SetPoint("RIGHT", prevname, "LEFT", -settings.hspacing, 0)
			end
			button:SetFrameLevel(frame:GetFrameLevel() + (DL_UNIT_BUFFCOUNT+1 - i))
			button:SetWidth(settings.size)
			button:SetHeight(settings.size)
			buttontexture:SetWidth(settings.size)
			buttontexture:SetHeight(settings.size)
		end
	else
		local button = _G[frame:GetName().."_1"]
		local buttontexture = _G[frame:GetName().."_1_Texture"]
		button:ClearAllPoints()
		button:SetPoint("TOPLEFT", frame:GetName(), "TOPLEFT", 0, 0)
		button:SetFrameLevel(frame:GetFrameLevel() + 1)
		button:SetWidth(settings.size)
		button:SetHeight(settings.size)
		buttontexture:SetWidth(settings.size)
		buttontexture:SetHeight(settings.size)
		local prevname
		for i=2,DL_UNIT_BUFFCOUNT do
			prevname = frame:GetName().."_"..(i-1)
			button = _G[frame:GetName().."_"..i]
			buttontexture = _G[frame:GetName().."_"..i.."_Texture"]
			button:ClearAllPoints()
			if (settings.vertical) then
				button:SetPoint("TOP", prevname, "BOTTOM", 0, -settings.vspacing)
			else
				button:SetPoint("LEFT", prevname, "RIGHT", settings.hspacing, 0)
			end
			button:SetFrameLevel(frame:GetFrameLevel() + i)
			button:SetWidth(settings.size)
			button:SetHeight(settings.size)
			buttontexture:SetWidth(settings.size)
			buttontexture:SetHeight(settings.size)
		end
	end
	if (settings.tworows) then
		local row2start = ceil(settings.shown / 2) + 1
		if (settings.reverse) then
			row2start = DL_UNIT_BUFFCOUNT - row2start + 1
		end
		local button = _G[frame:GetName().."_"..row2start]
		button:ClearAllPoints()
		if (settings.vertical) then
			if (settings.reverse) then
				button:SetPoint("RIGHT", frame:GetName().."_"..DL_UNIT_BUFFCOUNT, "LEFT", -settings.hspacing, 0)
			else
				button:SetPoint("LEFT", frame:GetName().."_1", "RIGHT", settings.hspacing, 0)
			end
		else
			if (settings.reverse) then
				button:SetPoint("BOTTOM", frame:GetName().."_"..DL_UNIT_BUFFCOUNT, "TOP", 0, settings.vspacing)
			else
				button:SetPoint("TOP", frame:GetName().."_1", "BOTTOM", 0, -settings.vspacing)
			end
		end
	end
	local stacksize = 9 * settings.size / 15
	for i=1,DL_UNIT_BUFFCOUNT do
		local button = _G[frame:GetName().."_"..i]
		local stack = button.Stack
		local scale = button:GetScale()
		stack:SetFont("Interface/AddOns/DiscordUnitFrames/CustomFonts/DUF_Font1.ttf", stacksize, "outline='THICK'")
		button:SetScale(scale + 1)
		button:SetScale(scale)
	end
	for i=1,DL_UNIT_BUFFCOUNT do
		local button = _G[frame:GetName().."_"..i]
		local duration = _G[frame:GetName().."_"..i].Duration
		duration:SetFont(NS:GetResourcePath(settings.durationfont, true), settings.durationheight)
		duration:SetTextColor(settings.durationcolor.r, settings.durationcolor.g, settings.durationcolor.b, settings.durationalpha)
		local scale = button:GetScale()
		button:SetScale(scale + 1)
		button:SetScale(scale)
		duration:ClearAllPoints()
		duration:SetPoint("TOP", button, "BOTTOM", settings.durationx, settings.durationy)
		if (settings.showduration) then
			duration:Show()
		else
			duration:Hide()
		end
		button.playerbuff = 1
	end
	NS:DUF_UnitFrame_InitBuffs(self, unit)
end

function DUF_Initialize_ClassIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_ClassIcon"]
	local texture = _G[DUF_FRAME_DATA[unit].frame.."_ClassIcon"].Icon
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].ClassIcon
	NS:DUF_Initialize_Element(unit, frame, settings)
	texture:SetTexture("Interface/Glues/CharacterCreate/UI-CharacterCreate-Classes")
	NS:DUF_Set_ClassIcon(unit)
end

function DUF_Initialize_TargetMarkers()
	local f = DUF_TargetFrame_Markers
	local settings = DUF_Settings[DUF_INDEX].target.Markers
	NS:DUF_Initialize_Element("target", f, settings)
	f.hidewhensolo = settings.hidewhensolo
	for i=1,8 do
		local frame = _G["DUF_TargetFrame_Markers_" .. i]
		frame:SetSize(settings.markersize, settings.markersize)
		local texture = _G["DUF_TargetFrame_Markers_" .. i .. "_Texture"]
		texture:SetAllPoints()
		frame:ClearAllPoints()
		frame:SetPoint("TOPLEFT", f, "TOPLEFT", settings.xos[i], settings.yos[i])
		texture:SetTexture(settings.icons[i].file)
		texture:SetTexCoord(settings.icons[i].left, settings.icons[i].right, settings.icons[i].top, settings.icons[i].bottom)
	end
	if f.hidewhensolo then
		f:Hide()
	end
end

function DUF_Initialize_QuestBossIcon()
	local f = DUF_TargetFrame_QuestBossIcon
	local settings = DUF_Settings[DUF_INDEX].target.QuestBossIcon
	f.questbossiconhide = settings.hide
	NS:DUF_Initialize_Element("target", f, settings)
	f:SetSize(settings.size, settings.size)
	local texture = DUF_TargetFrame_QuestBossIcon.Icon
	texture:SetTexture(settings.texture)
	if DUF_OPTIONS_VISIBLE then
		f:SetAlpha(1)
	else
		f:SetAlpha(0)
	end
end

function DUF_Initialize_Debuffs(self, unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_Debuffs"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].Debuffs
	NS:DUF_Initialize_Element(unit, frame, settings)
	if (settings.reverse) then
		local button = _G[frame:GetName().."_"..DL_UNIT_BUFFCOUNT]
		local buttontex = _G[frame:GetName().."_"..DL_UNIT_BUFFCOUNT.."_Texture"]
		button:ClearAllPoints()
		button:SetPoint("BOTTOMRIGHT", frame:GetName(), "BOTTOMRIGHT", 0, 0)
		button:SetFrameLevel(frame:GetFrameLevel() + 1)
		button:SetWidth(settings.size)
		button:SetHeight(settings.size)
		buttontex:SetWidth(settings.size)
		buttontex:SetHeight(settings.size)
		local prevname
		for i=DL_UNIT_BUFFCOUNT-1, 1, -1 do
			prevname = frame:GetName().."_"..(i+1)
			button = _G[frame:GetName().."_"..i]
			buttontex = _G[frame:GetName().."_"..i.."_Texture"]
			button:ClearAllPoints()
			if (settings.vertical) then
				button:SetPoint("BOTTOM", prevname, "TOP", 0, settings.vspacing)
			else
				button:SetPoint("RIGHT", prevname, "LEFT", -settings.hspacing, 0)
			end
			button:SetFrameLevel(frame:GetFrameLevel() + (DL_UNIT_BUFFCOUNT + 1 - i))
			button:SetWidth(settings.size)
			button:SetHeight(settings.size)
			buttontex:SetWidth(settings.size)
			buttontex:SetHeight(settings.size)
		end
	else
		local button = _G[frame:GetName().."_1"]
		local buttontex = _G[frame:GetName().."_1_Texture"]
		button:ClearAllPoints()
		button:SetPoint("TOPLEFT", frame:GetName(), "TOPLEFT", 0, 0)
		button:SetFrameLevel(frame:GetFrameLevel() + 1)
		button:SetWidth(settings.size)
		button:SetHeight(settings.size)
		buttontex:SetWidth(settings.size)
		buttontex:SetHeight(settings.size)
		local prevname
		for i=2,DL_UNIT_BUFFCOUNT do
			prevname = frame:GetName().."_"..(i-1)
			button = _G[frame:GetName().."_"..i]
			buttontex = _G[frame:GetName().."_"..i.."_Texture"]
			button:ClearAllPoints()
			if (settings.vertical) then
				button:SetPoint("TOP", prevname, "BOTTOM", 0, -settings.vspacing)
			else
				button:SetPoint("LEFT", prevname, "RIGHT", settings.hspacing, 0)
			end
			button:SetFrameLevel(frame:GetFrameLevel() + i)
			button:SetWidth(settings.size)
			button:SetHeight(settings.size)
			buttontex:SetWidth(settings.size)
			buttontex:SetHeight(settings.size)
		end
	end
	if (settings.flash) then
		for i=1,DL_UNIT_BUFFCOUNT do
			local button = _G[frame:GetName().."_"..i]
			button.flashing = true
			button.timer = .5
			button.direction = 1
		end
	else
		for i=1,DL_UNIT_BUFFCOUNT do
			local button = _G[frame:GetName().."_"..i]
			button:SetAlpha(1)
			button.flashing = nil
		end
	end
	if (settings.tworows) then
		local row2start = ceil(settings.shown / 2) + 1
		if (settings.reverse) then
			row2start = DL_UNIT_BUFFCOUNT - row2start + 1
		end
		local button = _G[frame:GetName().."_"..row2start]
		button:ClearAllPoints()
		if (settings.vertical) then
			if (settings.reverse) then
				button:SetPoint("RIGHT", frame:GetName().."_"..DL_UNIT_BUFFCOUNT, "LEFT", -settings.hspacing, 0)
			else
				button:SetPoint("LEFT", frame:GetName().."_1", "RIGHT", settings.hspacing, 0)
			end
		else
			if (settings.reverse) then
				button:SetPoint("BOTTOM", frame:GetName().."_"..DL_UNIT_BUFFCOUNT, "TOP", 0, settings.vspacing)
			else
				button:SetPoint("TOP", frame:GetName().."_1", "BOTTOM", 0, -settings.vspacing)
			end
		end
	end
	local stacksize = 12 * settings.size / 15
	for i=1,DL_UNIT_BUFFCOUNT do
		local button = _G[frame:GetName().."_"..i]
		local stack = button.Stack
		local scale = button:GetScale()
		stack:SetFont("Interface\\AddOns\\DiscordUnitFrames\\CustomFonts\\DUF_Font1.ttf", stacksize, "outline='THICK'")
		button:SetScale(scale + 1)
		button:SetScale(scale)
	end
	for i=1,DL_UNIT_BUFFCOUNT do
		local button = _G[frame:GetName().."_"..i]
		local duration = _G[frame:GetName().."_"..i].Duration
		duration:SetFont(NS:GetResourcePath(settings.durationfont, true), settings.durationheight)
		duration:SetTextColor(settings.durationcolor.r, settings.durationcolor.g, settings.durationcolor.b, settings.durationalpha)
		local scale = button:GetScale()
		button:SetScale(scale + 1)
		button:SetScale(scale)
		duration:ClearAllPoints()
		duration:SetPoint("TOP", button, "BOTTOM", settings.durationx, settings.durationy)
		if (settings.showduration) then
			duration:Show()
		else
			duration:Hide()
		end
		button.playerbuff = 1
	end
	NS:DUF_UnitFrame_InitDebuffs(self, unit)
end

function NS:DUF_Initialize_Element(unit, frame, settings, debug)
	frame.timer = nil
	DUF_Initialize_BaseSettings(frame, settings, debug)
	NS:DUF_Set_ElementPosition(frame, DUF_FRAME_DATA[unit].frame, settings)
	SetupClickTargetting(frame, settings)
end

function DUF_Initialize_EliteTexture()
	local frame = DUF_TargetFrame_EliteTexture
	local settings = DUF_Settings[DUF_INDEX].target.EliteTexture
	NS:DUF_Initialize_Element("target", frame, settings)
	NS:DUF_Set_EliteTexture()
-- BFA	DUF_TargetFrame:RegisterUnitEvent("PLAYER_TARGET_CHANGED", "player", "target")
	DUF_TargetFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function DUF_Initialize_Frame(self, unit)
	local unitFrame = _G[DUF_FRAME_DATA[unit].frame]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]
	DUF_Initialize_BaseSettings(unitFrame, settings)
	DUF_Initialize_FrameLoc(unit)
	unitFrame:SetScale(settings.scale)
	unitFrame.scale = settings.scale
	unitFrame.basecolor = {}
	unitFrame.basecolor.r = settings.bgcolor.r
	unitFrame.basecolor.g = settings.bgcolor.g
	unitFrame.basecolor.b = settings.bgcolor.b
	if ((not UnitName(unit)) and (not UnitExists(unit)) and (not DUF_OPTIONS_VISIBLE)) then
		unitFrame:Hide()
	end
end

function DUF_Initialize_FrameLoc(unit)
	local unitFrame = _G[DUF_FRAME_DATA[unit].frame]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index]
	unitFrame:ClearAllPoints()
	if (settings.connect and unitFrame:GetID() > 1) then
		if (DUF_FRAME_DATA[unit].index == "party" or DUF_FRAME_DATA[unit].index == "partypet" or DUF_FRAME_DATA[unit].index == "boss") then
			local connectframe = "DUF_PartyFrame"..(unitFrame:GetID() - 1)
			if (DUF_FRAME_DATA[unit].index == "partypet") then
				connectframe = "DUF_PartyPetFrame"..(unitFrame:GetID() - 1)
			end
			if (DUF_FRAME_DATA[unit].index == "boss") then
				connectframe = "DUF_BossFrame"..(unitFrame:GetID() - 1)
			end
			if (settings.connectmethod == 1) then
				unitFrame:SetPoint("TOP", connectframe, "BOTTOM", 0, -settings.spacing)
			elseif (settings.connectmethod == 2) then
				unitFrame:SetPoint("BOTTOM", connectframe, "TOP", 0, settings.spacing)
			elseif (settings.connectmethod == 3) then
				unitFrame:SetPoint("LEFT", connectframe, "RIGHT", settings.spacing, 0)
			elseif (settings.connectmethod == 4) then
				unitFrame:SetPoint("RIGHT", connectframe, "LEFT", -settings.spacing, 0)
			end
		else
			unitFrame:SetPoint("TOPLEFT", "UIParent", "TOPLEFT", settings.xos[unitFrame:GetID()], settings.yos[unitFrame:GetID()])
		end
	else
		unitFrame:SetPoint(settings.attachpoint[unitFrame:GetID()], UIParent, settings.attachto[unitFrame:GetID()], settings.xos[unitFrame:GetID()], settings.yos[unitFrame:GetID()])
	end
end

--[[ Unanimated version
function DUF_Initialize_HappinessIcon()
	local settings = DUF_Settings[DUF_INDEX].pet.HappinessIcon
	NS:DUF_Initialize_Element("pet", DUF_PetFrame_HappinessIcon, settings)
	DUF_Initialize_IconBackground(DUF_PetFrame_HappinessIcon, settings)
	DUF_PetFrame_HappinessIcon_Texture:SetTexture("Interface\\PetPaperDollFrame\\UI-PetHappiness")
	NS:DUF_HappinessIcon_Update()
end
]]--

function DUF_Initialize_HappinessIcon()
	local settings = DUF_Settings[DUF_INDEX].pet.HappinessIcon
	NS:DUF_Initialize_Element("pet", DUF_PetFrame_HappinessIcon, settings)
	DUF_Initialize_IconBackground(DUF_PetFrame_HappinessIcon, settings)
	DUF_PetFrame_HappinessIcon_Texture:SetTexture("Interface\\PetPaperDollFrame\\UI-PetHappiness")
	DUF_PetFrame_HappinessIcon.animatePetUnhappy = settings.animatePetUnhappy
	if settings.animatePetUnhappy then
		if not DUF_PetFrame_HappinessIcon.AnimationGroup1 then
			DUF_PetFrame_HappinessIcon.AnimationGroup1 = DUF_PetFrame_HappinessIcon:CreateAnimationGroup()
			DUF_PetFrame_HappinessIcon.AnimationGroup1:SetToFinalAlpha(true)
			DUF_PetFrame_HappinessIcon.AnimationGroup1:SetLooping("REPEAT")

			local anim = DUF_PetFrame_HappinessIcon.AnimationGroup1:CreateAnimation("Alpha")
			anim:SetChildKey("Icon")
			anim:SetOrder(1)
			anim:SetDuration(2)
			anim:SetFromAlpha(1)
			anim:SetToAlpha(0.3)
			anim:SetSmoothing("NONE")

			anim = DUF_PetFrame_HappinessIcon.AnimationGroup1:CreateAnimation("Scale")
			anim:SetChildKey("Icon")
			anim:SetOrder(1)
			anim:SetDuration(2)
			if anim.SetFromScale then
				anim:SetFromScale(1, 1)
			elseif anim.SetScaleFrom then
				anim:SetScaleFrom(1, 1)
			end
			if anim.SetToScale then
				anim:SetToScale(0.6, 0.6)
			elseif anim.SetScaleTo then
				anim:SetScaleTo(0.6, 0.6)
			end
			anim:SetSmoothing("NONE")

			anim = DUF_PetFrame_HappinessIcon.AnimationGroup1:CreateAnimation("Alpha")
			anim:SetChildKey("Icon")
			anim:SetOrder(2)
			anim:SetDuration(2)
			anim:SetFromAlpha(0.3)
			anim:SetToAlpha(1)

			anim = DUF_PetFrame_HappinessIcon.AnimationGroup1:CreateAnimation("Scale")
			anim:SetChildKey("Icon")
			anim:SetOrder(2)
			anim:SetDuration(2)

			if anim.SetFromScale then
				anim:SetFromScale(1, 1)
			elseif anim.SetScaleFrom then
				anim:SetScaleFrom(1, 1)
			end
			if anim.SetToScale then
				anim:SetToScale(1.55, 1.55)
			elseif anim.SetScaleTo then
				anim:SetScaleTo(1.55, 1.55)
			end
			anim:SetSmoothing("NONE")
		end
		DUF_PetFrame_HappinessIcon.AnimationGroup1:Stop()
	end		
	NS:DUF_HappinessIcon_Update()
end

function DUF_Initialize_LeaderIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_LeaderIcon"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].LeaderIcon

	NS:DUF_Initialize_Element(unit, frame, settings)
	DUF_Initialize_IconBackground(frame, settings)
	_G[DUF_FRAME_DATA[unit].frame.."_LeaderIcon"].Icon:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
	NS:DUF_LeaderIcon_Update(unit)
end

function DUF_Initialize_LootIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_LootIcon"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].LootIcon

	NS:DUF_Initialize_Element(unit, frame, settings)
	DUF_Initialize_IconBackground(frame, settings)
	_G[DUF_FRAME_DATA[unit].frame.."_LootIcon"].Icon:SetTexture("Interface\\GroupFrame\\UI-Group-MasterLooter")
	NS:DUF_LootIcon_Update(unit)
end

function DUF_Initialize_Portrait(unit)
	DUF_CreatePortrait(unit)
	local use3D, use3Dface = NS:DUF_Get3D(unit)
	local frame
	local unitFrame = _G[DUF_FRAME_DATA[unit].frame]
	unitFrame.use3Dface = use3Dface
	unitFrame.p3d = nil
	unitFrame.p2d = nil
	unitFrame.pInit = true
	local p2d = _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame2d"]
	local p3d = _G[DUF_FRAME_DATA[unit].frame .. "_Portrait_Frame3d"]
	if (use3D) then
		unitFrame.p3d = true
		frame = p3d
		if p2d then
			p2d:Hide()
		end
	else
		unitFrame.p2d = true
		frame = p2d
		if p3d then
			p3d:Hide()
		end
	end
	_G[DUF_FRAME_DATA[unit].frame .. "_Portrait"] = frame
	_G[frame:GetName().."_Border"]:Hide()
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].Portrait
	NS:DUF_Initialize_Element(unit, frame, settings)
	if (not use3D) then	
		if (settings.showborder) then
			_G[frame:GetName().."_Border"]:Show()
		else
			_G[frame:GetName().."_Border"]:Hide()
		end
	end
	_G[frame:GetName().."_Border"]:SetVertexColor(settings.bordcolor.r, settings.bordcolor.g, settings.bordcolor.b)
	NS:DUF_Set_Portrait(unit)
end

function DUF_Initialize_PVPIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_PVPIcon"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].PVPIcon
	NS:DUF_Initialize_Element(unit, frame, settings)
	DUF_Initialize_IconBackground(frame, settings)
	_G[DUF_FRAME_DATA[unit].frame.."_PVPIcon"].Icon:SetTexCoord(0, .6875, 0, .625)
	NS:DUF_PVPIcon_Update(unit)
end

function DUF_Initialize_RaceIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_RaceIcon"]
	local texture = _G[DUF_FRAME_DATA[unit].frame.."_RaceIcon"].Icon
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].RaceIcon
	NS:DUF_Initialize_Element(unit, frame, settings)
	texture.targetIcon = nil
	if (settings.targetIcon) then
		if (frame:GetParent().targetIcon == 1) then
			frame:GetParent().targetIcon = 3
		else
			frame:GetParent().targetIcon = 2
		end
		texture.targetIcon = 1
		texture:Hide()
		frame:GetParent().targetIndex = nil
		texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	end
	NS:DUF_Set_RaceIcon(unit)
end

function DUF_Initialize_RankIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_RankIcon"]
	local texture = _G[DUF_FRAME_DATA[unit].frame.."_RankIcon_Texture"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].RankIcon
	NS:DUF_Initialize_Element(unit, frame, settings)
	texture.targetIcon = nil
	if (settings.targetIcon) then
		if (frame:GetParent().targetIcon == 2) then
			frame:GetParent().targetIcon = 3
		else
			frame:GetParent().targetIcon = 1
		end
		texture.targetIcon = 1
		texture:Hide()
		frame:GetParent().targetIndex = nil
		texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	end
	NS:DUF_Set_RankIcon(unit)
end

function DUF_Initialize_RoleIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_RoleIcon"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].RoleIcon
	NS:DUF_Initialize_Element(unit, frame, settings)
	DUF_Initialize_IconBackground(frame, settings)
	NS:DUF_RoleIcon_Update(unit)
end

function DUF_Initialize_StatusBar(unit, bar, classSettings, debug)
	local settings, frame
	if classSettings then
		settings = classSettings
		frame = DUF_PlayerFrame_ClassFrameStatusFrame
	else
		settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusBar[bar]
		frame = _G[DUF_FRAME_DATA[unit].frame..DUF_STATUSBARS[bar]]
	end
	if not settings or not frame then return end
	frame.stacked = settings.stacked
	local statusbar = frame.Bar1
	local statusbarbg = frame.Bar2
	local texture = ""
	if (not settings.resizemax) then
		frame.dynamicsize = nil
	end
	NS:DUF_Initialize_Element(unit, frame, settings, debug)
	if (settings.customtexture and settings.customtexture ~= "") then
		statusbar:SetTexture(NS:GetResourcePath(settings.customtexture))
	else
		statusbar:SetTexture("Interface\\TargetingFrame\\UI-StatusBar")
	end
	if (settings.customtexture2 and settings.customtexture2 ~= "") then
		statusbarbg:SetTexture(NS:GetResourcePath(settings.customtexture2))
	else
		statusbarbg:SetTexture("Interface\\AddOns\\DiscordLibrary\\PlainBackdrop")
	end
	statusbar:SetWidth(settings.width)
	statusbarbg:SetWidth(settings.width)
	statusbar:SetHeight(settings.height)
	statusbarbg:SetHeight(settings.height)
	statusbar:ClearAllPoints()
	statusbarbg:ClearAllPoints()
	statusbar.direction = settings.direction
	statusbarbg.direction = settings.direction
	if (settings.direction == 1) then
		statusbar:SetPoint("LEFT", frame, "LEFT", 0, 0)
		if frame.stacked then
			statusbarbg:SetPoint("LEFT", statusbar, "RIGHT", 0, 0)
		else
			statusbarbg:SetPoint("LEFT", frame, "LEFT", 0, 0)
		end
		statusbar.length = settings.width
		statusbarbg.length = settings.width
	elseif (settings.direction == 2) then
		statusbar:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		if frame.stacked then
			statusbarbg:SetPoint("RIGHT", statusbar, "LEFT", 0, 0)
		else
			statusbarbg:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		end
		statusbar.length = settings.width
		statusbarbg.length = settings.width
	elseif (settings.direction == 3) then
		statusbar:SetPoint("BOTTOM", frame, "BOTTOM", 0, 0)
		if frame.stacked then
			statusbarbg:SetPoint("BOTTOM", statusbar, "TOP", 0, 0)
		else
			statusbarbg:SetPoint("BOTTOM", frame, "BOTTOM", 0, 0)
		end
		statusbar.length = settings.height
		statusbarbg.length = settings.height
	else
		statusbar:SetPoint("TOP", frame, "TOP", 0, 0)
		if frame.stacked then
			statusbarbg:SetPoint("TOP", statusbar, "BOTTOM", 0, 0)
		else
			statusbarbg:SetPoint("TOP", frame, "TOP", 0, 0)
		end
		statusbar.length = settings.height
		statusbarbg.length = settings.height
	end

--	if (bar ~= 2 and bar ~= 5 and bar ~= 6 and bar ~= 7) then  -- Fixed 5.3
--		statusbar:SetVertexColor(settings.color.r, settings.color.g, settings.color.b)
--	end
	if (bar ~= 2 and bar ~= 5 and bar ~= 7) then  -- ignore manabar (unit, unittarget and altmana)colours as they will be set seperately
		statusbar:SetVertexColor(settings.color.r, settings.color.g, settings.color.b)
	end
	statusbarbg:SetVertexColor(settings.color2.r, settings.color2.g, settings.color2.b)
	local StatusBarFrame = _G[DUF_FRAME_DATA[unit].frame]
	if (bar == 1) then
		StatusBarFrame.lasthealth = UnitHealth(unit)
		StatusBarFrame.lasthealthalpha = 0
		NS:DUF_HealthBar_Update(StatusBarFrame)
	elseif (bar == 2) then
		StatusBarFrame.lastmana = UnitPower(unit)
		StatusBarFrame.lastmanaalpha = 0
		StatusBarFrame.manatype = 0
		NS:DUF_ManaBar_Update(StatusBarFrame)
		NS:DUF_ManaBar_UpdateType(StatusBarFrame)
	elseif (bar == 3) then
		if (unit == "player") then
			NS:DUF_XPBar_Update()
		else
			NS:DUF_PetXPBar_Update()
		end
	elseif (bar == 4) then
		NS:DUF_TargetHealthBar_Update(StatusBarFrame)
	elseif (bar == 5) then
		NS:DUF_TargetManaBar_Update(StatusBarFrame)
		NS:DUF_TargetManaBar_UpdateType(StatusBarFrame)
	elseif (bar == 6) then  -- Fixed 5.3
		NS:DUF_CastBar_Update(StatusBarFrame)
	elseif (bar == 7) then   -- Fixed 5.3
		local powerType = UnitPowerType("player")
		if unit == "player" and (NS.DUF_ISDRUID or NS.ALTPOWERTYPES[powerType] or DUF_OPTIONS_VISIBLE) then
			StatusBarFrame.lastmana = UnitPower(unit)
			StatusBarFrame.lastmanaalpha = 0
			NS:DUF_AltManaBar_Update(StatusBarFrame)
			StatusBarFrame.manatype = 0
			DUF_AltManaBar_UpdateType(StatusBarFrame)
		end
		NS:DUF_AltManaBar_Update_Display()
	elseif (bar == 8) then   -- Honor bar
		NS:DUF_HonorBar_Update()
	end
end

function DUF_Initialize_StatusIcon(unit)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_StatusIcon"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].StatusIcon
	NS:DUF_Initialize_Element(unit, frame, settings)
	DUF_Initialize_IconBackground(frame, settings)
	NS:DUF_StatusIcon_Update(unit)
end

function DUF_Initialize_PhaseIcon(unit)
	local icon = _G[DUF_FRAME_DATA[unit].frame.."_PhaseIcon"]
	local texture = _G[icon:GetName().."_Texture"]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].PhaseIcon
	NS:DUF_Initialize_Element(unit, icon, settings)
	DUF_Initialize_IconBackground(icon, settings)
	texture:SetTexture("Interface\\Addons\\DiscordUnitFrames\\CustomTextures\\Phased")
	NS:DUF_PhaseIcon_Update(unit)
end

function DUF_Initialize_TextBox(unit, box)
	local frame = _G[DUF_FRAME_DATA[unit].frame.."_TextBox_"..box]
	local settings = DUF_Settings[DUF_INDEX][DUF_FRAME_DATA[unit].index].TextBox[box]
	if ((not frame) or (not settings)) then return end
	local fontstring = _G[DUF_FRAME_DATA[unit].frame.."_TextBox_"..box].TextString
	fontstring:SetText(nil)
	NS:DUF_Initialize_Element(unit, frame, settings)
	local fontFlags = ""
	if (settings.outline) then
		fontFlags = "outline='"..settings.outline.."'"
	end
	fontstring:SetFont(NS:GetResourcePath(settings.font, true), settings.textheight, fontFlags)
--	frame:SetScale(2)
	frame:SetScale(1)
	fontstring:SetJustifyH(settings.justify)
	fontstring:SetJustifyV(settings.justifyV)
	fontstring:SetTextColor(settings.textcolor.r, settings.textcolor.g, settings.textcolor.b)
	for variable, value in pairs(NS.DUF_VARIABLE_FUNCTIONS) do
		for _,event in pairs(value.events) do
			frame:UnregisterEvent(event)			
		end
	end
	if (settings.text and settings.text ~= "") then
		frame.variables = {}
		for variable, value in pairs(NS.DUF_VARIABLE_FUNCTIONS) do
			if (strfind(settings.text, variable)) then
				frame.variables[variable] = true
				for _,event in pairs(value.events) do
					frame:RegisterEvent(event)			
				end			
			end
			if (DUF_UNIT_CHANGED_EVENTS[frame:GetParent().unit]) then
				frame:RegisterEvent(DUF_UNIT_CHANGED_EVENTS[frame:GetParent().unit])
			end
		end
		NS:DUF_TextBox_Update(frame)
	else
		fontstring:SetText("")
		frame:Hide()
	end
	frame.formatH = settings.formatH
	frame.shortHealth = settings.shortHealth
	frame.formatM = settings.formatM
	frame.shortMana = settings.shortMana
	frame.checkname = nil
	frame.checkhealth = nil
	frame.checkmana = nil
	frame.checkhealthmax = nil
	frame.checkmanamax = nil
	frame.checklevel = nil
	frame.checktype = nil
	frame.checkoffline = nil
	frame.checkreaction = nil
	frame.checkcombat = nil
	frame.checkpetxp = nil
	frame.checkvisibility = nil
	frame.checkcolor = nil
	if (strfind(unit, 'boss')) then
		if (strfind(settings.text, '$h') or strfind(settings.text, '$kh')) then
			frame.checkbosshealth = true
		end
	end
	if (strfind(settings.text, '$t')) then
		frame.checkname= true
	end
	if (strfind(settings.text, '$th')) then
		frame.checkhealth = true
	end
	if (strfind(settings.text, '$tm')) then
		frame.checkmana = true
	end
	if (strfind(settings.text, '$tx')) then
		frame.checkhealthmax = true
	end
	if (strfind(settings.text, '$ta')) then
		frame.checkhealth = true
		frame.checkhealthmax = true
	end
	if (strfind(settings.text, '$tb')) then
		frame.checkmana = true
		frame.checkmanamax = true
	end
	if (strfind(settings.text, '$ty')) then
		frame.checkmanamax = true
	end
	if (strfind(settings.text, '$tl')) then
		frame.checklevel = true
	end
	if (strfind(settings.text, '$tt')) then
		frame.checktype = true
	end
	if (strfind(settings.text, '$re')) then
		frame.checkreaction = true
	end
	if (strfind(settings.text, '$of')) then
		frame.checkoffline = true
	end
	if (strfind(settings.text, '$ic')) then
		frame.checkcombat = true
	end
	if (strfind(settings.text, '$iv')) then
		frame.checkvisibility = true
	end
	if (strfind(settings.text, '$px') or strfind(settings.text, '$py') or strfind(settings.text, '$pc') or strfind(settings.text, '$pp') or strfind(settings.text, '$pg')) then
		frame.checkpetxp = true
	end
	if (strfind(settings.text, '$cr') or strfind(settings.text, '$cw') or strfind(settings.text, '$cm') or strfind(settings.text, '$ch') or strfind(settings.text, '$cq')) then
		frame.checkcolor = true
	end
end
